package common.btv;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.channels.FileChannel;
import java.util.HashMap;
import java.util.Map;

import common.ts.TsConstants;

import demux.ts.TsInfo;
import demux.ts.TsRawPacketPart;

public class TpParser implements TsConstants {
	private static final int BTV_META_PID=0x1129;
	public static final String UNIQUE_CHANNEL_ID="Unique-Channel-ID";
	//private File file;
	private ByteBuffer metaBuffer;
	private boolean snipped=false;
	
	public TpParser(File file) throws IOException {
		FileChannel fc=new RandomAccessFile(file, "r").getChannel();
		ByteBuffer buffer=ByteBuffer.allocate(TS_PACKET_SIZE);
		TsRawPacketPart packetPart=new TsRawPacketPart(0);
		
		for (long start=fc.size() - TS_PACKET_SIZE * 2;start > 0;start -=TS_PACKET_SIZE) {
			fc.position(start);
			buffer.rewind();
			fc.read(buffer);
			
			packetPart.setByteBuffer(buffer);
			TsInfo tsInfo=packetPart.getTsInfo();
			if (tsInfo.pid==BTV_META_PID && tsInfo.pidStart) {
				long metaLen=fc.size() - start;
				fc.position(start);
				metaBuffer=ByteBuffer.allocate((int)metaLen);
				fc.read(metaBuffer);
				fc.position(start - TS_PACKET_SIZE);
				buffer.rewind();
				fc.read(buffer);
				snipped=(buffer.get(0)==TS_SYNC_BYTE);
				break;
			}
		}
		
		fc.close();
	}
	
	public Map parse() {
		//metaBuffer.position(TS_PACKET_SIZE);//Size the 0s and the first packet
		metaBuffer.rewind();
		ByteBuffer tmpBuffer=ByteBuffer.allocate((metaBuffer.capacity()/TS_PACKET_SIZE) * (TS_PACKET_SIZE-4));
		while (metaBuffer.remaining() > 0) {
			int header=metaBuffer.getInt();
			//Copy payload
			for (int i=0;i<(TS_PACKET_SIZE-4)/8;i++) {
				tmpBuffer.putLong(metaBuffer.getLong());
			}
		}
		tmpBuffer.position(0);
		tmpBuffer.order(ByteOrder.LITTLE_ENDIAN);
		int len=tmpBuffer.getInt();
		tmpBuffer.limit(len + 4);
		int u1=tmpBuffer.getInt(); // Entries
		HashMap map=new HashMap();
		
		while (tmpBuffer.remaining() > 0) {
			int keyLen=tmpBuffer.getInt();
			if (keyLen < 6) {
				break;
			}
			char c1=tmpBuffer.getChar();
			char c2=tmpBuffer.getChar();
			char c3=tmpBuffer.getChar();
			keyLen -=6;
			StringBuffer key=new StringBuffer(keyLen/2);
			while (keyLen > 2) {
				key.append(tmpBuffer.getChar());
				keyLen -=2;
			}
			char eos=tmpBuffer.getChar();
			int valueLen=tmpBuffer.getInt();
			StringBuffer value=new StringBuffer(valueLen/2);
			while (valueLen > 2) {
				value.append(tmpBuffer.getChar());
				valueLen -=2;
			}
			eos=tmpBuffer.getChar();
			map.put(key.toString(), value.toString());
		}
		return map;
	}
	public static void main(String args[]) {
		String fileName="M:/tools/My Name Is Earl-(Stole a Motorcycle)-2008-04-10-0.ts";
		try {
			TpParser tpParser=new TpParser(new File(fileName));
			System.out.println(tpParser.parse());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public ByteBuffer getMetaBuffer() {
		return metaBuffer;
	}

	public boolean isSnipped() {
		return snipped;
	}
}
