package common.ts;

import java.io.FileInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.HashMap;

import common.DirectMemoryManager;
import common.StreamDescriptor;

import demux.ts.TsRawPacketPart;

public class ProgramAssociationTable extends TsTable implements TsTableParsedListener {
	public static final short NETWORK_PROGRAM_ID=0;
	public static final String NETWORK_INFORMATION="Network Information";
	public static final HashMap<Integer, StreamDescriptor> DUMMY_SD_MAP=new HashMap<Integer, StreamDescriptor>();

	private short selectedProgramNumbers[];
	private ProgramInfo programInfo[];
	private TableHandler tableHandler;
	
	public ProgramAssociationTable(TableHandler tableHandler) {
		super(0, TsConstants.PROGRAM_ASSOCIATION_TABLE_ID);
		this.tableHandler=tableHandler;
		//This will spawn up the PMTs
		this.addListener(this);
	}
	/**
	 * @return Array of programNumber and programPid.  programPid needs to be & with 0x1fff
	 */
	public TsRawPacketPart[]  parse() {
		ByteBuffer tmp=mergeIn(buffer);
		int len=getLen(buffer);
		int programs=(len-13)/4;
		programInfo=new ProgramInfo[programs];
		tmp.position(9);
		for (int i=0;i<programs;i++) {
			programInfo[i]=new ProgramInfo(tmp.getShort(), tmp.getShort() & 0x1fff);
			if (programInfo[i].getProgramNumber()==NETWORK_PROGRAM_ID) {
				programInfo[i].setStreamDescriptorMap(DUMMY_SD_MAP);
			}
		}
		
		found=true;
		notifyListeners();
		return buffer;
	}
	public TsRawPacketPart[] filter(boolean reparse) {
		if (out==null || reparse) {
			ByteBuffer tmp=mergeIn(buffer);
			int outLen=13 + selectedProgramNumbers.length*4;
			out=DirectMemoryManager.allocateDirect(outLen);
			//int len=getLen();
			//tmp.limit(9);
			//out.put(tmp);
			//tmp.limit(len);
			copy(tmp, out, 9);
			for (int i=0;tmp.remaining() > 4;i++) {
				short programNumber=tmp.getShort();
				short programPid=tmp.getShort();

				for (int j=0;j<selectedProgramNumbers.length;j++) {
					if (programNumber==selectedProgramNumbers[j]) {
						out.putShort(programNumber);
						out.putShort(programPid);
						break;
					}
				}
			}
			finalizeOut();
		}
		return mergeOut(buffer);
		
	}
	public ProgramInfo[] getProgramInfo() {
		return programInfo;
	}
	public static void main(String args[]) {
		try {
			FileChannel in=new FileInputStream("m:/tools/My Name Is Earl-(Guess Who's Coming Out of Joy)-2007-08-23-0s.ts").getChannel();
			ByteBuffer buffer=in.map(FileChannel.MapMode.READ_ONLY,0,188);
			
			ProgramAssociationTable pat=new ProgramAssociationTable(new TableHandler());
			
			pat.append(new TsRawPacketPart(buffer));
			pat.parse();
			ProgramInfo programInfo[]=pat.getProgramInfo();
			short programNumbers[]=new short[programInfo.length];
			for (int i=0;i < programInfo.length;i++) {
				programNumbers[i]=programInfo[i].getProgramNumber();
			}
			/*TsRawPacketPart out[]=pat.filter(programNumbers, true);
			ByteBuffer test=out[0].getByteBuffer();
			for (int i=0;i<buffer.capacity();i++) {
				if (buffer.get(i) != test.get(i)) {
					System.out.println("Failed at: " + i + " - " + Integer.toString(buffer.get(i) & 0xff, 16) + ':' + Integer.toString(test.get(i) & 0xff, 16));
				}
			}*/
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	public void tsTableDidParse(TsTable tsTable, TsRawPacketPart buffer[]) {
		//Spawn up PMTs
		if (tsTable==this) {
			for (int i=0;i<programInfo.length;i++) {
				if (programInfo[i].getProgramNumber() != NETWORK_PROGRAM_ID) {
					tableHandler.add(new ProgramMapTable(programInfo[i]));
				}
			}
		}
	}
	public void setSelectedProgramNumbers(short[] selectedProgramNumbers, TableHandler tableHandler) {
		this.selectedProgramNumbers = selectedProgramNumbers;
	}
	public TableHandler getTableHandler() {
		return tableHandler;
	}

}
