package demux.pes;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;

import muxer.MuxManager;
import muxer.filter.PesStreamIdentifierMuxFilter;
import muxer.ts.TsPesMuxer;

import org.xml.sax.SAXException;

import common.FileStream;
import common.Job;
import common.JobStreamI;
import common.Logger;
import common.StreamDescriptor;
import common.mpeg.MpegConstants;
import common.packet.ClockReferenceI;
import common.packet.PacketPart;
import common.packet.PesPacket;
import common.packet.PesPacketPart;

import demux.Demuxer;

public class DvdDemuxer extends Demuxer {
	private static final int CHUNK_SIZE=2048;
	private Logger logger=Logger.getLogger(this.getClass());
//	private int index=0;
	
	private class StreamHandler {
		private PesPacket packet;
		private ArrayList<ByteBuffer> slices=new ArrayList<ByteBuffer>();
		
		private StreamHandler(PesPacket packet) {
			this.packet=packet;
		}
	}
	
	public void parse(JobStreamI stream, MuxManager muxManager) throws IOException {
		int index=0;
		HashMap<Integer, Object> streamPacketMap=new HashMap<Integer, Object>();
		Map<Integer, StreamDescriptor> muxedStreamDescriptorMap=muxManager.getMuxedStreamDescriptorMap();
		if (muxedStreamDescriptorMap==null) {
			//Hack to prevent NPE on stream scan
			muxedStreamDescriptorMap=new HashMap<Integer, StreamDescriptor>();
		}
		
		long audio=0, video=0;
		
		stream.setChunkSize(CHUNK_SIZE);
		int rc=MuxManager.CONTINUE;
		stream.open();
		while (!stream.isEOS() && rc!=MuxManager.ABORT) {
			int streamId=stream.peek(3)&0xff;
			if (streamId==0xBA) {
				stream.skip(14);//Skip the SCR
				continue;
			}
			boolean aligned=false;
			if (stream.peek(0)==0 && stream.peek(1)==0) {
				if (stream.peek(2)==1) {
					aligned=true;
				} else if (stream.peek(2)!=0) {
					logger.error("PES Stream Error.  Attemping to recover...");
				}
			} else {
				logger.error("PES Stream Error.  Attemping to recover...");
			}
			while (!aligned) {
				stream.skip(1);
				aligned=stream.peek(0)==0 && stream.peek(1)==0 && stream.peek(2)==1;
			}
			
			int packetLen=getShort(4, stream);
			//int packetStart=fileBuffer.position();
			int packetEnd=packetLen+6;
			//System.out.println("streamId: " + Integer.toHexString(streamId));
			if (streamId==0xFD || streamId==0xE2 || streamId==0xE0 || streamId==0xc0) {
				video+=packetLen;
				int flags=stream.peek(7)&0xff;
				int ptsDtsFlag=flags&0xC0;
				int offset=9;
				
				if (ptsDtsFlag==0xC0) {
					offset+=10;
				} else if (ptsDtsFlag==0x80) {
					offset+=5;
				}
				if ((flags & 0x3E)!=0) {
					System.out.println("Doah! unchecked flags: " + Integer.toHexString(flags & 0x3E));
				}
				//TODO: Add code for other flags
				if ((flags&0x01)==0x01) {
					int extensionFlags=stream.peek(offset)&0xff;
					offset++;
					if ((extensionFlags & 0x10)==0x10) {
						offset+=2;
					}
					if ((extensionFlags & 0xE0) != 0) {
						System.out.println("Doah! unchecked extension flags: " + Integer.toHexString(flags & 0xE0));
					}
					if ((extensionFlags & 0x01)==0x01) {
						offset++; //Size;
						streamId = (streamId <<8) + (stream.peek(offset)&0xFF);
					}
				}
				if (isCopied(streamId)) {
					int headerLen=(stream.peek(8) & 0xff) + 9;
					
					//This code combines the packet into one super packet
					Integer streamIdInteger=Integer.valueOf(streamId);
					//PesPacket packet=(PesPacket)streamPacketMap.get(streamIdInteger);
					StreamHandler streamHandler=(StreamHandler)streamPacketMap.get(streamIdInteger);
					//PesPacketPart header;
					//PacketPart payload;
					if (streamHandler != null) {
						//payload=packet.getPayload();
						if (ptsDtsFlag > 0 /*|| payload.getLen() > 0x8000*/) {
							flushPacket(streamHandler, muxManager);
							streamHandler=null;
						}
					}
					if (streamHandler==null) {

						PesPacket packet=new PesPacket(streamId, index++);
						PesPacketPart header=muxManager.getPesPacketPart(headerLen);
						packet.setHeader(header);
						header.appendFrom(stream, headerLen);
						streamHandler=new StreamHandler(packet);
						streamPacketMap.put(streamIdInteger, streamHandler);
					} else {
						stream.skip(headerLen);//Discard blank header
					}
					int payloadLen = packetLen + 6 - headerLen;
					streamHandler.slices.add(stream.slice(payloadLen));
					//payload.appendFrom(stream, payloadLen);
				} else {
					stream.skip(packetEnd);
				}
			} else if (streamId==0xBD) {
				audio+=packetLen;
				int headerLen=(stream.peek(8) & 0xff) + 9;
				//Get the substreamId
				streamId=stream.peek(headerLen) & 0xff;
				//System.out.println("SubStream: " + Integer.toHexString(streamId));
				if (isCopied(streamId)) {
					Integer streamIdInteger=Integer.valueOf(streamId);
					int payloadLen = packetLen + 6 - headerLen;
					
					PesPacket packet=new PesPacket(streamId, index++);
					PesPacketPart header=new PesPacketPart(stream.slice(headerLen));
					//header.appendFrom(stream, headerLen);
					packet.setHeader(header);
					if ((streamId >= 0xC0) || (streamId >= 0x80 && streamId < 0xB0)) {
						PesPacket oldPacket=(PesPacket)streamPacketMap.get(streamIdInteger);
						int headers=stream.peek(1) & 0xFF;
						int headerOffset;
						if (headers > 0) {
							headerOffset=getShort(2, stream) - 1;
						} else {
							headerOffset=0;
						}
						if (headerOffset > payloadLen) {
							System.out.println("Doah! HeaderOffset greater than payload len");
						}
						stream.skip(4);
						payloadLen-=4;
						byte syncBytes[]={stream.peek(headerOffset), stream.peek(headerOffset + 1)};
						if (oldPacket !=null) {
							PacketPart oldPayload=oldPacket.getPayload();
							//oldPayload.getByteBuffer().put();
							oldPayload.appendFrom(stream.slice(headerOffset));
							muxManager.addPacket(oldPacket);
							header.setReadWrite();
							header.set16(header.limit() + oldPayload.limit() - 6, 4);
							payloadLen -=headerOffset;
						} 
						long pts=header.getClockReference(ClockReferenceI.CLOCK_REFERENCE_TYPE_PTS);
						while (headers > 1) {
							//Find next sync
							int i=2;
							while (true) {
								if (stream.peek(i)==syncBytes[0] && stream.peek(i+1)==syncBytes[1]) {
									break;
								}
								i++;
							}
							PacketPart payload=muxManager.getPacketPart(i);
							payload.appendFrom(stream, i);
							payloadLen-=i;
							packet.setPayload(payload);
							flushPacket(packet, muxManager);
							headers--;
							packet = new PesPacket(streamId, index++);

							StreamDescriptor streamDescriptor=muxedStreamDescriptorMap.get(streamIdInteger);
							if (streamDescriptor != null && streamDescriptor.getFrameTime()!=StreamDescriptor.VALUE_UNKNOWN) {
								header=muxManager.getPesPacketPart(MpegConstants.PTS_PES_HEADER.length);
								header.appendFrom(MpegConstants.PTS_PES_HEADER, 0, MpegConstants.PTS_PES_HEADER.length);
								pts+=streamDescriptor.getFrameTime();
								header.offsetClock(pts);//This works because the current time is 0
							} else {
								/* This code should be right, but doesn't always work ? */
								header=muxManager.getPesPacketPart(MpegConstants.EMPTY_PES_HEADER.length);
								header.appendFrom(MpegConstants.EMPTY_PES_HEADER, 0, MpegConstants.EMPTY_PES_HEADER.length);
							}
							header.setByte(MpegConstants.PRIVATE_PACKET, 3);
							packet.setHeader(header);

						}
						PacketPart payload=muxManager.getPacketPart(payloadLen);
						payload.appendFrom(stream, payloadLen);
						packet.setPayload(payload);
						streamPacketMap.put(streamIdInteger,packet);
						
					} else if (streamId <0x80) {
						//Subtitles
						PacketPart payload=muxManager.getPacketPart(payloadLen-1);
						stream.skip(1);
						payload.appendFrom(stream, payloadLen-1);
						
						packet.setHeader(header);
						packet.setPayload(payload);
						muxManager.addPacket(packet);
					} else if (streamId >= 0xB0 && streamId < 0xC0) {
						//True HD
						PacketPart payload=muxManager.getPacketPart(payloadLen-5);
						stream.skip(5);
						payload.appendFrom(stream, payloadLen-5);
						
						packet.setHeader(header);
						packet.setPayload(payload);
						muxManager.addPacket(packet);

					} else {
						//Unknown
						PacketPart payload=muxManager.getPacketPart(payloadLen);
						payload.appendFrom(stream, payloadLen);
						
						packet.setHeader(header);
						packet.setPayload(payload);
						muxManager.addPacket(packet);
					}
				} else {
					stream.skip(packetEnd);
				}
			} else {
				stream.skip(packetEnd);
			}
		}
		for (Iterator i=streamPacketMap.values().iterator();i.hasNext();) {
			Object o=i.next();
			if (o instanceof StreamHandler) {
				flushPacket((StreamHandler)o, muxManager);
			} else {
				flushPacket((PesPacket)o, muxManager);
			}
		}
		muxManager.streamEnded();
	}
	public Map<Integer, StreamDescriptor> getStreamDescriptorMap(JobStreamI stream, MuxManager muxManager) throws IOException {
		PesStreamIdentifierMuxFilter identifier = new PesStreamIdentifierMuxFilter();
		muxManager.add(identifier);
		stream.setStreamSize(Math.min(stream.getStreamSize(),200*1024*1024));
		muxManager.mux(stream);
		muxManager.remove(identifier);
		stream.close();
		Map<Integer, StreamDescriptor> identifierMap=identifier.getStreamDescriptorMap();
		for (Iterator<StreamDescriptor> i=identifierMap.values().iterator();i.hasNext();) {
			StreamDescriptor streamDescriptor=i.next();
			if (streamDescriptor.getStreamId() >=0x20 && streamDescriptor.getStreamId() <0x30 && streamDescriptor.getCodec()==StreamDescriptor.CODEC_UNKNOWN) {
				streamDescriptor.setClassification(StreamDescriptor.CLASSIFICATION_SUBTITLE);
				streamDescriptor.setCodec("Subtitle");
				streamDescriptor.setTsType(MpegConstants.TS_STREAM_TYPE_SUBTITLE);
			}
		}
		if (stream instanceof FileStream) {
			FileStream fileStream=(FileStream)stream;
			File file=fileStream.getFiles()[0];
			String fileName=file.getName().toUpperCase();
			if (fileName.endsWith(".EVO")) {
				//Todo, deal with multiple XPL
				File xplFile=new File(file.getParentFile(),"../ADV_OBJ/VPLST000.XPL");
				if (xplFile.canRead()) {
					try {
						XplHelper helper=new XplHelper(xplFile);
						helper.setLanguage(file, identifierMap);
					} catch (ParserConfigurationException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (SAXException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}
		}
		return identifierMap;
	}
	private int getShort(int offset, JobStreamI stream) throws IOException {
		return ((stream.peek(offset) & 0xff)<<8) + (stream.peek(offset+1) & 0xff);		
	}
	private int flushPacket(StreamHandler streamHandler, MuxManager muxManager) throws IOException {
		final ArrayList<ByteBuffer> slices=streamHandler.slices;
		final int len=slices.size();
		int size=0;
		for (int i=0;i<len;i++) {
			size+=slices.get(i).capacity();
		}
		PesPacketPart packetPart=muxManager.getPesPacketPart(size);
		streamHandler.packet.setPayload(packetPart);
		for (int i=0;i<len;i++) {
			ByteBuffer buffer=(ByteBuffer)slices.get(i);

			packetPart.appendFrom(buffer);
		}
		return flushPacket(streamHandler.packet, muxManager);
	}
	private int flushPacket(PesPacket packet, MuxManager muxManager) throws IOException {
		PesPacketPart header=packet.getHeader();
		int len=header.limit() + packet.getPayload().limit() -6;
		if (len > PesUtil.MAX_PACKET_LEN) {
			len=0;
		}
		if (header.get16(4) != len) {
			header.setReadWrite();
			header.set16(len, 4);
		}
		return muxManager.addPacket(packet);
	}
	/*public static void main(String args[]) {
		//FileJob fileJob=new FileJob(null, new File[] {new File("H:/VIDEO_TS/VTS_06_1.VOB"),new File("H:/VIDEO_TS/VTS_06_2.VOB")});
		FileStream fileJob=new FileStream(new Job(null), new File[] {new File("J:/HVDVD_TS/FEATURE_1.EVO")});
		DvdDemuxer demuxer=new DvdDemuxer();
		MuxManager muxManager=new MuxManager(demuxer);
		try {
			Map streamDescriptorMap=muxManager.getStreamDescriptorMap(fileJob);
			System.out.println(streamDescriptorMap);
			ArrayList list=new ArrayList(2);
			list.add(streamDescriptorMap.get(Integer.valueOf(0xE0)));
			list.add(streamDescriptorMap.get(Integer.valueOf(0x80)));
			muxManager.add(new TsPesMuxer(new FileOutputStream(new File("C:/Walk the Line.ts")).getChannel()));
			muxManager.mux(fileJob);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}*/
}
