package demux.ts;

import java.nio.ByteBuffer;

import common.mpeg.MpegConstants;
import common.mpeg.MpegUtil;
import common.packet.PacketPart;
import common.ts.ProgramMapTable;

public class TsRawPacketPart extends PacketPart {
	
	private static final long PCR_FLAG=0x10;
	private static final int ADAPTATION_FLAG=0x20;
	private static final int PAYLOAD_FLAG=0x10;
	
	private TsInfo tsInfo;
	
	public TsRawPacketPart(int len) {
		super(len);
	}
	public TsRawPacketPart(ByteBuffer buffer) {
		super(buffer);
	}
	public TsRawPacketPart(byte buffer[]) {
		super(buffer);
		this.buffer.limit(buffer.length);
		this.buffer.position(buffer.length);
	}
	
	/**
	 * Returns the offset of the TS sync mark
	 * @return
	 */
	public int getTsSyncOffset() {
		return 0;
	}

	public int[][] getPmtPid(int offset) {
		return ProgramMapTable.getPmtPids(buffer, getTsSyncOffset()+4);
	}

	protected final long getSystemClockReference() {
		int offset=getTsSyncOffset();
		if (tsInfo.adaptationControl >= ADAPTATION_FLAG && buffer.get(offset+4) != 0 && (buffer.get(offset+5) & PCR_FLAG)==PCR_FLAG) {
			return MpegUtil.getPCRTime(buffer, getTsSyncOffset() + 6);
		} else {
			return CLOCK_REFERENCE_NA;
		}
	}

	public final void setSystemClockReference(long pcr) {
		int offset=getTsSyncOffset();
		MpegUtil.setPCRTime(buffer, offset + 6, pcr);
	}
	public TsInfo getTsInfo() {
		if (tsInfo==null) {
			int flags=buffer.getShort(getTsSyncOffset()+1) & 0xffff;
			byte byte3=buffer.get(getTsSyncOffset()+3);
			tsInfo=new TsInfo();
			if (!(tsInfo.error=(flags & MpegConstants.TRANSPORT_ERROR_INDICATOR)==MpegConstants.TRANSPORT_ERROR_INDICATOR)) {
				tsInfo.pid=flags & 0x1FFF;
				tsInfo.pidStart=(flags & MpegConstants.PID_START_FLAG)==MpegConstants.PID_START_FLAG;
				tsInfo.adaptationControl=byte3 & 0x30;
			}
		}
		return tsInfo;
	}
	public int getPesHeaderOffset() {
		int offset=getTsSyncOffset()+4;
		if (tsInfo.adaptationControl==(ADAPTATION_FLAG | PAYLOAD_FLAG)) {
			offset+=(buffer.get(offset) & 0xff) + 1;
		}
		return offset;
	}
	public long getClockReference(int type) {
		if ((type==CLOCK_REFERENCE_TYPE_PTS || type==CLOCK_REFERENCE_TYPE_DTS) && !tsInfo.pidStart) {
			return CLOCK_REFERENCE_NA;
		} else {
			return super.getClockReference(type);
		}
	}
	public boolean hasPesHeader() {
		return tsInfo.pidStart;
	}
	@Override
	public void setByteBuffer(ByteBuffer byteBuffer) {
		tsInfo=null;
		super.setByteBuffer(byteBuffer);
	}
}
