package io;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;

public class AsyncFileOutputStream extends OutputStream implements Runnable {
	private FileOutputStream out;
	private byte buffer[];
	private int len=0;
	private int writeLen;
	boolean running=true;
	private IOException lastException=null;
	//private int writes=0, background=0, force=0;
	
	public AsyncFileOutputStream(String fileName) throws FileNotFoundException {
		this(fileName, 16*1024, 4*1024);
	}
	public AsyncFileOutputStream(String fileName, int bufferSize, int maxWriteSize) throws FileNotFoundException {
		out=new FileOutputStream(fileName);
		buffer=new byte[bufferSize];
		writeLen=bufferSize-maxWriteSize;
		new Thread(this).start();
	}
	public void run() {
		while (running) {
			synchronized (buffer) {
				if (len > writeLen) {
					try {
						//background++;
						flush();
					} catch (IOException e) {
						lastException=e;
					}
				}
			}
			synchronized (this) {
				try {
					//System.out.println("Waiting...");
					wait();
				} catch (InterruptedException e) {
				}
			}
		}
	}
	public void write(byte inBuffer[], int offset, int len) throws IOException {
		if (lastException != null) {
			throw lastException;
		}
		synchronized (this.buffer) {
			if (this.len + len > buffer.length) {
				//force++;
				flush();
			}
			System.arraycopy(inBuffer, offset, buffer, this.len, len);
			this.len +=len;
		}
		synchronized (this) {
			this.notify();
		}
	}
	
	public void write(byte inBuffer[]) throws IOException {
		write(inBuffer, 0, inBuffer.length);
	}
	public void write(int b) {
		buffer[len++]=(byte)b;
		System.out.println("Call to write(int)");
		synchronized (this) {
			this.notify();
		}
	}
	public void flush() throws IOException {
		/*writes++;
		if (writes % 1000==0) {
			System.out.println("Background Percent " + ((float)background/(float)writes));
		}*/
		synchronized (buffer) {
			out.write(buffer, 0, len);
			len=0;
		}
	}
	
	public void close() throws IOException {
		running=false;
		synchronized (this) {
			this.notify();
		}
		flush();
		out.close();
		
	}
}
