package muxer.filter;

import java.io.IOException;
import java.nio.ByteBuffer;

import muxer.MuxFilter;

import common.BitStream;
import common.mpeg.MpegUtil;
import common.packet.Packet;
import common.packet.PacketPart;
import common.vc1.Vc1Constants;
import common.vc1.Vc1Deinterlacer;

public class Vc1HdDvdToBluRayMuxFilter extends MuxFilter implements Vc1Constants {
	//private static final int bottomMask[]={0,1,3,7,15,31,63,127};
	protected static final int LOWER_MASK[]={0,1,3,7,15,31,63,127,255};
	//private static final int binary[]={1,2,4,8,16,32,64,128};
	
	private int streamId;
	private Vc1Deinterlacer deinterlacer=new Vc1Deinterlacer();
	
	
	public int getPosition() {
		return MuxFilter.POSITION_POST_BUFFER;
	}
	public Vc1HdDvdToBluRayMuxFilter(int streamId) {
		this.streamId=streamId;
	}
	
	public int addPacket(Packet packet) throws IOException {
		if (packet.getStreamId()==streamId) {
			byte type=-1;
			int typeStart=-1;
			PacketPart packetPart=packet.getPayload();
			BitStream bitStream=new BitStream(packetPart.limit()*105/100);
			BitStream inBitStream=new BitStream(new byte[0]);
			ByteBuffer packetPartBuffer=packetPart.getByteBuffer();
			for (int start=0;start>=0;) {
				start=MpegUtil.findNextStartCode(packetPartBuffer, start);
				if (start>=0) {
					byte nextType=packetPart.get(start+3);
					if (nextType==PICTURE_START || nextType==ENTRY_POINT_START || nextType==SEQUENCE_START) {
						if (type==PICTURE_START) {
							ByteBuffer inByteBuffer=packetPart.getByteBuffer();
							inBitStream.setByteBuffer(inByteBuffer);
							inBitStream.setPosition(typeStart);//Clears the bitsUsed
							deinterlacer.copyPicture(start, inBitStream, bitStream);
						} else if (type==ENTRY_POINT_START) {
							deinterlacer.copyEntryPoint(typeStart, start, packetPart.getByteBuffer(), bitStream);
						} else if (type==SEQUENCE_START) {
							deinterlacer.copySequence(typeStart, start, packetPart.getByteBuffer(), bitStream);
						} 
						type=nextType;
						typeStart=start;
					}
					start+=4;
				}
			}
			inBitStream.setByteBuffer(packetPart.getByteBuffer());
			inBitStream.setPosition(typeStart);
			deinterlacer.copyPicture(packetPart.getByteBuffer().limit(), inBitStream, bitStream);
			packet.setPayload(new PacketPart(bitStream.getByteBuffer()));
		}
		return super.addPacket(packet);
	}

}
