package muxer.ts;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.channels.WritableByteChannel;

import muxer.MuxFilter;
import muxer.MuxManager;
import muxer.MuxUtil;

import common.packet.Packet;

import demux.ts.TsRawPacketPart;

public class TsRawMuxer extends MuxFilter {
	private TsPacketWriter packetWriter;
	
	public TsRawMuxer(File file) throws FileNotFoundException {
		this(new RandomAccessFile(file,"rw").getChannel());
	}
	public TsRawMuxer(WritableByteChannel out) {
		this(new TsPacketWriter(out));
		
	}
	public TsRawMuxer(TsPacketWriter packetWriter) {
		this.packetWriter=packetWriter;
	}
	public void init() throws IOException {
		packetWriter.setSyncStreamId(MuxUtil.getLinearStream(muxManager.getMuxedStreamDescriptorMap()).getStreamId());
	}
	public int addPacket(Packet packet) throws IOException {
		TsRawPacketPart packetPart=(TsRawPacketPart)packet.getPayload();
		try {
			packetWriter.writeTsPacket(packetPart);
		} catch (IOException ioe) {
			packetWriter.close();
			throw ioe;
		}
		//Can't call super because we may hold onto the payload
		//return super.addPacket(packet);
		return MuxManager.CONTINUE;
	}
	public void streamEnd() throws IOException {
		packetWriter.streamEnd();
		super.streamEnd();
	}
	public int getPosition() {
		return POSITION_MUXER;
	}	
	/*protected void write(TsRawPacketPart packetPart) throws IOException {
		packetPart.write(out, packetPart.getOffset(), packetPart.getLen()-packetPart.getOffset());
	}*/
}
